//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "SignalSynthCfg.h"
#include "Signal.h"
#include "Utils.h"
#include <SysUtils.hpp>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmSignalSynthConfig *frmSignalSynthConfig = NULL;
//---------------------------------------------------------------------------
__fastcall TfrmSignalSynthConfig::TfrmSignalSynthConfig(TComponent* Owner)
	: TForm(Owner),
	fBaseSamplingFreq(20000.0f),
	iMaxSamples(4096),
	confirmed(false)
{
	std::string sWaveType;
	for (int i=0; i<Signal::WAVE_LIMITER; i++)
	{
		if (Signal::GetTypeDescription(i, sWaveType) == 0)
			cbWaveType->Items->Add(sWaveType.c_str());
		else
			cbWaveType->Items->Add("");
	}
	cbWaveType->ItemIndex = 0;
	edFrequency->Text = 100.0f;
	edAmplitude->Text = AnsiString::FloatToStrF(1.00f, AnsiString::sffFixed, 3, 2);
	edOffset->Text = AnsiString::FloatToStrF(1.65f, AnsiString::sffFixed, 3, 2);
}

//---------------------------------------------------------------------------

void __fastcall TfrmSignalSynthConfig::btnApplyClick(TObject *Sender)
{
	float fAmpl, fFreq, fOffset;
	try
	{
		fAmpl = StrToFloat(edAmplitude->Text);
		fFreq = StrToFloat(edFrequency->Text);
		fOffset = StrToFloat(edOffset->Text);
	}
	catch (EConvertError &E)
	{
		ShowMessage("Invalid value");
		return;
	}
	if (fAmpl < 0 || fFreq < 0)
	{
		ShowMessage("Invalid value");
		return;
	}
	if (GetRealFreq(fFreq) == 0)
	{
    	return;
	}
	signal.shape = cbWaveType->ItemIndex;
	signal.fAmplitude = fAmpl;
	signal.fOffset = fOffset; 
	confirmed = true;
	Close();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmSignalSynthConfig::edFrequencyChange(TObject *Sender)
{
	float reqFreq;
	try
	{
		reqFreq = StrToFloat(edFrequency->Text);
	}
	catch (EConvertError &E)
	{
		edRealFreq->Text = "";
		return;
	}
	edRealFreq->Text =
		AnsiString::FloatToStrF(GetRealFreq(reqFreq), AnsiString::sffGeneral, 6, 4);
}
//---------------------------------------------------------------------------

float TfrmSignalSynthConfig::GetRealFreq(float reqFreq)
{
	if (reqFreq > fBaseSamplingFreq/2 || reqFreq < fBaseSamplingFreq/iMaxSamples)
	{
		return 0.0f;
	}
	period = fBaseSamplingFreq/reqFreq + 0.5f;
    return fBaseSamplingFreq/period;
}

short* TfrmSignalSynthConfig::Generate(float fDivY, int& count)
{
    count = period;
	return signal.Generate(period, fDivY);
}

void __fastcall TfrmSignalSynthConfig::FormShow(TObject *Sender)
{
	confirmed = false;	
}
//---------------------------------------------------------------------------

